import argparse
import shutil
from pathlib import Path

from . import WorkSpacer

template_position = Path(__file__).parent.joinpath('templates')

parser = argparse.ArgumentParser(
    description="Changes the workspace, based on what output your cursor is on."
)
parser.add_argument('-d', '--debug', action='store_true',
                    help='Turn on debug mode.')
default_home = Path().home().joinpath('.config').joinpath('sway').joinpath('outputs')

required_group = parser.add_argument_group('Required', '')
required_group.add_argument("-i", "--index", type=int, required=True,
                            help="The indexed workspace for the output where the cursor is currently located")
required_group.add_argument('-o', '--output-location',
                            default=default_home,
                            help='The dir for where the location output configurations are located')

shift_group = parser.add_argument_group('Shift', 'manipulate the active window')
shift_group.add_argument("-s", "--shift", action='store_true',
                         help="Moves the active window to the index workspace")
shift_group.add_argument('-k', '--keep-with-it', action='store_true',
                         help='Moves the active window to the index workspace, and moves with it')
parsed_args = parser.parse_args()


def main():
    if default_home == parsed_args.output_location:
        if not default_home.is_dir():
            default_home.mkdir(parents=True, exist_ok=True)
            shutil.copy(template_position.joinpath('controls'), default_home.joinpath('controls'))
            parsed_args[
                'only_send'] = 'Before using this tool remember to add include $HOME/.config/sway/outputs to your configuration'
            WorkSpacer(parsed_args, template_position)
            exit()
    elif not Path(parsed_args.output_location).is_dir():
        raise Exception('--output-location MUST be a directory.')

    WorkSpacer(parsed_args, template_position).run()


if __name__ == '__main__':
    main()



import re
import subprocess
from pathlib import Path

import pynput
from i3ipc import Connection


def get_active_outputs():
    ps = subprocess.Popen(('xrandr', '--listmonitors'), stdout=subprocess.PIPE)
    output = subprocess.check_output(('awk', "{print $4}"), stdin=ps.stdout)
    ps.wait()
    return [x for x in output.decode('UTF-8').split('\n') if x is not None and x != '']


names_for_outputs = r'(' + '|'.join(get_active_outputs()) + ')'


class WorkSpacer:

    def __init__(self, args, template_position):
        self.workspaces = None
        self.args = args
        self.outputs = None
        self.template_position = template_position
        try:
            self.sway = Connection()
            self.outputs = self.sway.get_outputs()
        except Exception as e:
            pass
        if 'only_send' in args:
            self._send_msg(args.only_send)

    def _send_msg(self, msg):
        self.sway.command(f'exec "notify-send \'{msg}\'"')

    def _create_default_config(self, path: Path, name):
        # Count number of configs
        num = sum(1 for _ in Path(self.args.output_location).glob('*')) - 1  # don't include controls
        path.write_text(
            self.template_position.joinpath('output').read_text()
                .replace('{name}', name).replace('{index}', str(num)))

    def run(self):
        name = self._get_workspace_from_courser_position()
        output_config = Path(self.args.output_location).joinpath(name)
        if not output_config.exists():
            self._create_default_config(output_config, name)
        workspaces = []

        for matchNum, match in enumerate(
                re.finditer(r'workspace (.*) output (.*)', output_config.read_text(), re.MULTILINE)
        ):
            workspaces.append(match.group(1))

        if self.args.shift:
            self.sway.command(
                f'move container to workspace {workspaces[self.args.index - 1]}'
            )
            if not self.args.keep_with_it:
                return
        self.sway.command(f'workspace {workspaces[self.args.index - 1]}')

    def _get_workspace_from_courser_position(self):
        self.mouse_position = pynput.mouse.Controller().position
        for output in self.outputs:
            width = output.__dict__["rect"].__dict__["width"]
            height = output.__dict__["rect"].__dict__["height"]
            x_offset = output.__dict__["rect"].__dict__["x"]
            y_offset = output.__dict__["rect"].__dict__["y"]

            if x_offset == 0 and y_offset == 0:
                if x_offset <= self.mouse_position[0] <= x_offset + width and y_offset <= \
                        self.mouse_position[1] <= y_offset + height:
                    return output.__dict__["name"]
            elif x_offset == 0:
                if x_offset <= self.mouse_position[0] <= x_offset + width and y_offset < \
                        self.mouse_position[1] <= y_offset + height:
                    return output.__dict__["name"]
            elif y_offset == 0:
                if x_offset < self.mouse_position[0] <= x_offset + width and y_offset <= \
                        self.mouse_position[1] <= y_offset + height:
                    return output.__dict__["name"]
            else:
                if x_offset < self.mouse_position[0] <= x_offset + width and y_offset < \
                        self.mouse_position[1] <= y_offset + height:
                    return output.__dict__["name"]



from setuptools import setup
with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='sway-smart-workspace',
    version='0.0.5',
    packages=['sway_smart_workspace'],
    url='https://github.com/GeneralDenmark/PyOutputHandler',
    license='GNU Lesser General Public License v3 or later (LGPLv3+)',
    install_requires=[
        "evdev==1.3.0",
        "i3ipc==2.2.1",
        "pynput==1.7.1",
        "python-xlib==0.27",
        "six==1.15.0"
    ],
    entry_points={"console_scripts": ["sway-smart-workspace=sway_smart_workspace.__main__:main"]},
    scripts=["sway_smart_workspace/__main__.py"],
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Asger Geel Weirsøe',
    author_email='asger@weirsoe.dk',
    description='Simple program that looks through the sway config and finds the bound workspaces for each output, and then opening that workspace on the output, that the mouse is currently on.',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: Desktop Environment :: Window Managers",
    ],
)
